@@ -1,50 +1,42 @@
-{-# OPTIONS_GHC -fno-warn-orphans #-}
-{-# LANGUAGE CPP                  #-}
-{-# LANGUAGE PackageImports       #-}
+{-# OPTIONS_GHC -fno-warn-orphans -fno-warn-warnings-deprecations #-}
+{-# LANGUAGE CPP            #-}
+{-# LANGUAGE PackageImports #-}
 
-module Snap.Internal.Instances where
+-- Note re: "-fno-warn-warnings-deprecations" above: transformers has
+-- deprecated Control.Monad.Trans.Error (which we like) but we are going to
+-- provide an ErrorT instance for compatibility until the deprecated modules
+-- are removed.
 
-import           Control.Applicative
-import           Control.Monad.CatchIO ()
-import           Control.Monad.Cont
-import           Control.Monad.Error
-import           Control.Monad.List
-import           Control.Monad.RWS.Strict    hiding (pass)
-import qualified Control.Monad.RWS.Lazy      as LRWS
-import           Control.Monad.Reader
-import           Control.Monad.State.Strict
-import qualified Control.Monad.State.Lazy    as LState
-import           Control.Monad.Writer.Strict hiding (pass)
-import qualified Control.Monad.Writer.Lazy   as LWriter
-#if !MIN_VERSION_base(4,6,0)
-import           Prelude                     hiding (catch)
+module Snap.Internal.Instances () where
+------------------------------------------------------------------------------
+import           Control.Monad.Trans.Class         (MonadTrans (lift))
+import           Control.Monad.Trans.Error         (Error, ErrorT)
+#if MIN_VERSION_transformers(0,4,0)
+import           Control.Monad.Trans.Except        (ExceptT)
+#endif
+import           Control.Monad.Trans.List          (ListT)
+import           Control.Monad.Trans.Reader        (ReaderT)
+import qualified Control.Monad.Trans.RWS.Lazy      as LRWS (RWST)
+import           Control.Monad.Trans.RWS.Strict    (RWST)
+import qualified Control.Monad.Trans.State.Lazy    as LState (StateT)
+import           Control.Monad.Trans.State.Strict  (StateT)
+import qualified Control.Monad.Trans.Writer.Lazy   as LWriter (WriterT)
+import           Control.Monad.Trans.Writer.Strict (WriterT)
+import           Snap.Internal.Core                (MonadSnap (..))
+#if !MIN_VERSION_base(4,8,0)
+import           Data.Monoid                       (Monoid)
 #endif
-
 ------------------------------------------------------------------------------
-import           Snap.Internal.Types
-
-
-------------------------------------------------------------------------------
-instance MonadPlus m => MonadPlus (ContT c m) where
-    mzero = lift mzero
-    m `mplus` n = ContT $ \ f -> runContT m f `mplus` runContT n f
-
-
-------------------------------------------------------------------------------
-instance MonadPlus m => Alternative (ContT c m) where
-    empty = mzero
-    (<|>) = mplus
-
-
-------------------------------------------------------------------------------
-instance MonadSnap m => MonadSnap (ContT c m) where
-    liftSnap = lift . liftSnap
 
 
 ------------------------------------------------------------------------------
 instance (MonadSnap m, Error e) => MonadSnap (ErrorT e m) where
     liftSnap = lift . liftSnap
 
+#if MIN_VERSION_transformers(0,4,0)
+instance (MonadSnap m, Monoid e) => MonadSnap (ExceptT e m) where
+    liftSnap = lift . liftSnap
+#endif
 
 ------------------------------------------------------------------------------
 instance MonadSnap m => MonadSnap (ListT m) where
